//
//  MySpeexDecoderTest.m
//  SpeexCodec
//
//  Created by cliftoncraig07 on 3/16/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//
#import <SenTestingKit/SenTestingKit.h>
#import "MySpeexDecoder.h"

@interface MySpeexDecoderTest : SenTestCase
{
  MySpeexDecoder *decoder;
	NSFileManager *fileManager;
	NSString *speexFile;
	NSString *decodedOutput;
	
	BOOL willDecodeAudioWithAttributesWasCalled;
	NSDictionary *decodedAudioAttributes;
	BOOL audioWasDecodedWasCalled;
	NSMutableData *decodedAudio;
}

@end

@implementation MySpeexDecoderTest

-(void) setUp
{
	willDecodeAudioWithAttributesWasCalled = NO;
	audioWasDecodedWasCalled = NO;
	fileManager = [NSFileManager defaultManager];
  speexFile = [[NSBundle bundleForClass:[self class]] pathForResource:@"speex_output" ofType:@"spx"];
  STAssertNotNil(speexFile, @"Speex file required for the test!");
  decoder = [[MySpeexDecoder alloc] init];
	decodedOutput = [NSString stringWithFormat:@"%@/%@", [speexFile stringByDeletingLastPathComponent], @"decoded.raw"];
	decodedAudio =  [[NSMutableData alloc] init];
}

-(void) tearDown
{
	[decodedAudioAttributes release];
	[decodedAudio release];
	NSError *error = nil;
//	if([fileManager fileExistsAtPath:decodedOutput]) {
//		[fileManager removeItemAtPath:decodedOutput error:&error];
//		if(error) STFail([error description]);
//	}
}
-(void) assertDictionary:(NSDictionary*)aDictionary containsKey:(id)aKey withValue:(id)expectedValue
{
	id value = [aDictionary objectForKey:aKey];
	STAssertNotNil(value, @"Expecting a value for key %@", aKey);
	STAssertEqualObjects(value, expectedValue, @"Unexpected value for key");
}

-(void) testSpeex
{
	decoder.delegate = self;
	[decoder decodeAudioAtPath:speexFile];
	STAssertTrue(willDecodeAudioWithAttributesWasCalled, @"Expecting an event callback at the beginning of the decode.");
	STAssertNotNil(decodedAudioAttributes, @"expecting audio attibutes to be returned.");
	[self assertDictionary:decodedAudioAttributes containsKey:@"sampleRate" withValue:[NSNumber numberWithInt:16000]];
	[self assertDictionary:decodedAudioAttributes containsKey:@"frameSize" withValue:[NSNumber numberWithInt:320]];
	[self assertDictionary:decodedAudioAttributes containsKey:@"numberOfChannels" withValue:[NSNumber numberWithInt:1]];
	[self assertDictionary:decodedAudioAttributes containsKey:@"framesPerPacket" withValue:[NSNumber numberWithInt:1]];
	STAssertTrue(audioWasDecodedWasCalled, @"Should receive callbacks with decoded audio data");
	STAssertTrue([decodedAudio length] > 0, @"Expecting decoded audio to be accumulated.");
	[[NSFileManager defaultManager] createFileAtPath:decodedOutput contents:decodedAudio attributes:nil];
//	STFail(@"See file at %@", decodedOutput);
	STAssertEquals([decodedAudio length], (NSUInteger)74924 - 36, @"Roughly expecting a 73K audio file.");
}
MySpeexDecoder
#pragma mark MySpeexDecoderDelegate methods
-(void) willDecodeAudioWithAttributes:(NSDictionary*)attributes
{
	willDecodeAudioWithAttributesWasCalled = YES;
	decodedAudioAttributes = [attributes retain];
}

-(void) audioWasDecoded:(NSData*)rawAudio
{
	audioWasDecodedWasCalled = YES;
	[decodedAudio appendData:rawAudio];
}

@end
